--   This file is part of Reno desktop.
--
--  Reno desktop is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
--
--  Reno desktop is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License along with Reno desktop. If not, see <https://www.gnu.org/licenses/>. 
-- Client context menu base
local awmtk2 = require("awmtk2")
local wibox = require("wibox")
local gears = require("gears")
local awful = require("awful")
local beautiful = require("beautiful")
local menugen = require("context_menu")
local builder = require("builder")
local style = awmtk2.create_style("client_menu",
    awmtk2.generic.composite_widget,{})
local templates = awmtk2.create_template_lib("client_menu",awmtk2.templates,{})
local t = awmtk2.build_templates(templates,style)

-- Create a global context menu for clients first
-- This saves us memory on not creating separate menus for every client
if not context_menu then
local config_file = io.open(root_path.."/themes/"..global.theme..'/config/client_menu.json',"r")
local config 
if config_file then
    config = config_file:read("*a")
    config_file:close()
else
    config = [[{"list":[{"widget":"widgets.clientcontrols"}]}]]
end
context_menu = awful.popup(t.popup(builder(
    config,
    {
        style = style.base,
    }
)))
context_menu:connect_signal("button::press",function(self,x,y,b)
    if b == 3 then
        context_menu.visible = false
    end
end)
context_menu:connect_signal("property::visible",function(self,x,y,b)
    if not context_menu.visible then
        local children = context_menu.widget:get_children_by_id("menu_root")
        for k,v in pairs(children) do
            for k2,v2 in pairs(v.children) do
                v2:emit_signal("cascade::close")
            end
        end
    end
end)
client.connect_signal("focus",function()
    context_menu.visible = false
end)
for _,layout in pairs(context_menu.widget:get_children_by_id("menu_root")) do
    for _,button in pairs(layout.children) do
        button:connect_signal("cascade::kill",function()
            context_menu.visible = false
        end)
    end
end
end

return function(args)
    local widget = wibox.widget({
        widget = awful.widget.clienticon,
        client = args.client
    })
    widget:connect_signal("button::press",function()
        context_menu.visible = (not context_menu.visible)
        if context_menu.visible then 
            context_menu.x = mouse.coords().x
            context_menu.y = mouse.coords().y
        end
    end)
    return widget
end
